#------------------------------------------------------------------------------
#
#	Apple Macintosh Developer Technical Support
#
#	Program:	ProcDoggie
#	File:		ProcDoggie.make	- Make Source
#
#	Copyright  1988-1991 Apple Computer, Inc.
#	All rights reserved.
#
#------------------------------------------------------------------------------

AppName			=	ProcDoggie
Signature		=	'MOPD'

#------------------------------------------------------------------------------
# Options for our compilers:
#	-sym on: tells the compilers and linker to emit symbol information for
#		a source level debugger, such as SADE.
#	-mbg off: tells the compilers to not emit low-level debugger names. This
#		saves on file space, but you may wish to remove this option if you
#		need to debug with something like Macsbug.
#	-rd: for Rez means to suppress warnings for redeclared types (we redeclare
#		'RECT' because its not included in MPW 3.0).
#	-append: means to add the resources to the target file, rather than
#		deleting all the ones that are there first.
#	-d Signature...: is a way of passing our application's signature to Rez.
#		With this mechanism, we can define our signature here, and export
#		it to Rez, so that we don't have to declare it there, too.
#	-sn PASLIB=Main: puts all the routines that would normally go into the
#		PASLIB segment into the Main segment. This is done so that when we
#		call upon any low-level utilities, we don't potentially move memory
#		by loading in a segment.
#------------------------------------------------------------------------------

SymOptions		=	#-sym on					# turn this on to debug with SADE

POptions		=	{SymOptions} -mbg full
RezOptions		=	-rd -append -d Signature="{Signature}" -d AppName='"ProcDoggie"'
LinkOptions		=	{SymOptions} {SegmentMappings} -msg nodup
SegmentMappings	=	-sn INTENV=Main 
					-sn PASLIB=Main 
					-sn STDCLIB=Main 
					-sn SANELIB=Main 
					-sn MAFailureRes=Main 
					-sn Offscreen=Main 

#------------------------------------------------------------------------------
# These are modified default build rules.
#------------------------------------------------------------------------------
.p.o				.p
	{Pascal} {POptions} {DepDir}{Default}.p -o {TargDir}{Default}.p.o

.c.o				.c
	{C} {COptions} {DepDir}{Default}.c -o {TargDir}{Default}.c.o

#------------------------------------------------------------------------------
# These are the objects that we want to link with. If any one of these
# changes, then we invoke the Link command.
#------------------------------------------------------------------------------

AppObjects			=	
						ProcDoggie.p.o 
						UDialogUtils.p.o 
						UEmergMem.p.o 
						UMenuHandler.p.o 
						UProcessGuts.p.o 
						UProcessUtils.p.o 
						UGlobals.p.o

ProcessLDEFObjects	=	
						UProcessLDEF.p.o


#------------------------------------------------------------------------------
# Which files we link with depends on whether we are running MPW 3.1 or 3.2
# Under MPW 3.2 and later, CInterface.o and CRuntime.o are merged with 
# Interface.o and Runtime.o. The appropriate choice will be made 
# dynamically before compiling and linking occurs.  See ShellForce below.
#
# Be sure GestaltGlue.a.o is before Interface.o in PLibs and CLibs below.
#------------------------------------------------------------------------------
PLibs			=	
					"{Libraries}Runtime.o" 
					"{Libraries}Interface.o" 
					"{PLibraries}PasLib.o"

PLibs32			=	
					"{Libraries}Runtime.o" 
					"{Libraries}Interface.o" 
					"{PLibraries}PasLib.o"

CLibs			=	
					"{CLibraries}CRuntime.o" 
					"{CLibraries}CInterface.o" 
					"{Libraries}Interface.o"

CLibs32			=	
					"{CLibraries}StdCLib.o" 
					"{Libraries}Runtime.o" 
					"{Libraries}Interface.o"

#------------------------------------------------------------------------------
# Dependencies for the individual components. These will invoke the
# default build rules as described in Chapter 9 of the MPW 3.0 manual.
#------------------------------------------------------------------------------

{AppObjects}				{AppName}.make

ProcDoggie.p.o				ProcDoggie.p 
							UGlobals.p 
							UEmergMem.p 
							UMenuHandler.p 
							UProcessGuts.p 
							UProcessUtils.p

UGlobals.p.o				UGlobals.p UGlobals.inc1.p 
							UEmergMem.p

UDialogUtils.p.o			UDialogUtils.p UDialogUtils.inc1.p 
							UEmergMem.p

UEmergMem.p.o				UEmergMem.p UEmergMem.inc1.p

UMenuHandler.p.o			UMenuHandler.p UMenuHandler.inc1.p 
							UGlobals.p 
							UProcessGuts.p 
							UProcessUtils.p

UProcessGuts.p.o			UProcessGuts.p UProcessGuts.inc1.p 
							UGlobals.p 
							UDialogUtils.p 
							UEmergMem.p 
							UMenuHandler.p 
							UProcessLDEF.p 
							UProcessUtils.p

UProcessUtils.p.o			UProcessUtils.p UProcessUtils.inc1.p

UWindowHandler.p.o			UWindowHandler.p UWindowHandler.inc1.p 
							UGlobals.p 
							UEmergMem.p 
							UProcessGuts.p 
							UProcessUtils.p

{ProcessLDEFObjects}		{AppName}.make

UProcessLDEF.p.o			UProcessLDEF.p UProcessLDEF.inc1.p

#------------------------------------------------------------------------------
# Build rule that links our application together. If any of our objects 
# changes, or this makefile changes, then we relink.  The dummy prerequisite
# ShellForce must come before any other prerequisites for {AppName}
#------------------------------------------------------------------------------

{AppName}			{AppObjects}
	Link {LinkOptions} -o {Targ} {AppObjects} {PLibs32}
	SetFile {Targ} -t APPL -c {Signature} -a BM

{AppName}			{ProcessLDEFObjects}
	Link  -m PROCESSLIST -rn -rt LDEF=128 -o {Targ} {ProcessLDEFObjects}
	SetFile {Targ} -t APPL -c {Signature} -a BM

#------------------------------------------------------------------------------
# Build rule that creates our resources and adds them to the application
#------------------------------------------------------------------------------

{AppName}			{AppName}.make 
					{AppName}.r 
					{AppName}.rsrc
	Rez {RezOptions} {AppName}.r -o {Targ}
